@@ -218,11 +218,15 @@ class DomainCalculus:
 
             for index_2 in range(index_1 + 1, len(tc_attributes)):
                 value_2 = tc_attributes[index_2]
+
                 dc_var_2 = value_2[0]
                 tc_var_2 = value_2[1][0]
                 tc_column_2 = value_2[1][1]
 
+                print(f'EQUALS {dc_var_1} = {dc_var_2}')
+
                 if dc_var_1 == dc_var_2:
+                    print(f'EQUALS ADDED {dc_var_1} = {dc_var_2}')
                     equals.append(tc.Equals((tc_var_1, tc_column_1), (tc_var_2, tc_column_2)))
 
         #print(f'\nEQUALS: {equals}')
@@ -806,8 +810,6 @@ class Or(Formula):
 class Not(Formula):
     def __init__(self, child) -> None:
         super().__init__([child])
-        # not supported, since it doesn't make any sense
-        assert not isinstance(child, Tuple)
 
     def __repr__(self) -> str:
         return f'\\neg({self.children[0]})'
@@ -848,46 +850,55 @@ class Not(Formula):
 
 # temporary test to check the latex output
 if __name__ == '__main__':
-    add_table_scheme('Hauptstadt', ['name', 'bundesland'])
-    add_table_scheme('Bundesland', ['land', 'name'])
-    add_table_scheme('Stadt', ['name', 'istHauptstadt'])
+    print('TESTING \n\n')
 
-    t = Variable(name='tk')
-    x = Variable('x')
-    y = Variable('y')
-    z = Variable('z')
+    add_table_scheme('Tier', ['ChipID', 'Name', 'Spezies', 'Alter'])
+    add_table_scheme('Gehege', ['GehegeNR', 'Sicherheitsstufe', 'AnzahlTiere'])
+    add_table_scheme('Tierpfleger', ['MitarbeiterNR', 'Name', 'Berufserfahrung'])
+    add_table_scheme('zustaendig', ['MitarbeiterNR', 'GehegeNR', 'ChipID', 'Datum', 'VonUhrzeit', 'BisUhrzeit'])
 
-    eq = Equals('5', t)
+    MitarbeiterNR = Variable("MitarbeiterNR")
+    Name = Variable("Name")
+    bE = Variable("bE")
+    Alter = Variable("Alter")
+    ChipID = Variable("ChipID")
+    gNR = Variable("gNR")
+    vU1 = Variable("vU1")
+    bU1 = Variable("bU1")
+    vU2 = Variable("vU2")
+    bU2 = Variable("bU2")
+    Datum1 = Variable("Datum1")
+    Datum2 = Variable("Datum2")
 
-    tuple1 = Tuple('HAUPTSTADT', [t, z])
-    tuple2 = Tuple('BUNDESLAND', [t, 'Bayern'])
-    tuple3 = Tuple('STADT', [t, y])
+    expression = Exists(bE, Exists(Alter, Exists(ChipID, Exists(gNR, Exists(vU1, Exists(bU1, Exists(vU2, Exists(bU2, Exists(Datum1, Exists(Datum2,
+        And(
+            Tuple("Tierpfleger", [MitarbeiterNR, Name, bE]),
+            And(
+                GreaterThan(bE, 4),
+                And(
+                    Tuple("Tier", [ChipID, "Günther", "Nasenbär", Alter]),
+                    And(
+                        Tuple("zustaendig", [MitarbeiterNR, gNR, ChipID, Datum1, vU1, bU1]),
+                        And(
+                            Tuple("zustaendig", [MitarbeiterNR, gNR, ChipID, Datum2, vU2, bU2]),
+                            Not(
+                                Equals(
+                                    Datum1, Datum2
+                                )
+                            )
+                        )
+                    )
+                )
+            )
+        )
+    ))))))))))
 
-    and2 = And(GreaterThan(y, 5), tuple3)
-    fa1 = Forall(y, and2)
-    or1 = Or(tuple2, fa1)
-    ex2 = Exists(x, or1)
-    and1 = And(tuple1, ex2)
-    final = Exists(z, and1)
+    domainc = DomainCalculus(Result([MitarbeiterNR, Name]), expression)
 
-    result = Result([t])
-    domain_calculus = DomainCalculus(result, final)
 
-    # print(domain_calculus.to_TupleCalculus())
-
-    # print('\n\nEND DEV \n')
-
-    print(domain_calculus)
-    print('')
-    print(domain_calculus.to_sql())
-    print('')
-    print(domain_calculus.to_tuple_calculus())
-    print('')
-    print(domain_calculus.to_tuple_calculus().to_sql())
-
-    # print("\nPNF:\n")
-
-    # pnf = domain_calculus.to_normal_form()
-    # print(pnf)
-    # print('')
-    # print(domain_calculus.to_sql())
\ No newline at end of file
+    print(domainc)
+    print(domainc.to_sql())
+    print('\n')
+    tc = domainc.to_tuple_calculus()
+    print(tc)
+    print(tc.to_sql())
\ No newline at end of file
