@@ -1,8 +1,6 @@
 from abc import ABC, abstractclassmethod
 import py_compile
 
-from numpy import isin
-
 
 class TupleCalculus:
     def __init__(self, result, formula) -> None:
@@ -27,6 +25,7 @@ class TupleCalculus:
         return f'\\{{{self.result} \\vert {self.formula}\\}}'
 
     def to_sql(self) -> str:
+
         # Get Types of return variables
         variables = self.result.all_attributes + \
             list(self.result.selected_attributes.keys())
@@ -43,18 +42,32 @@ class TupleCalculus:
                 for attribute in attributes:
                     select_query += f'{variable.name}."{attribute}", '
             else:
-                raise Exception('The selected attributes must consist of a variable and either a attribute as a string or a list of attributes each as a string!')
+                raise Exception(
+                    'The selected attributes must consist of a variable and either a attribute as a string or a list of attributes each as a string!')
 
         select_query = select_query[:-len(', ')]
 
+        global declared_variables
+        declared_variables = list()
+        where_query = self.formula.to_sql() # this also initializes declared_variables
+
         from_query = ''
         for variable in variables:
+            if variable in declared_variables:
+                declared_variables.remove(variable)
+            else:
+                raise Exception(f'The variable {variable} was not declared.')
+
             from_query += f'{variable.type} {variable.name}, '
         from_query = from_query[:-len(', ')]
 
+        if len(declared_variables) > 0:
+            raise Exception(
+                f'The variables {declared_variables} should not be declared.')
+
         query = f'SELECT DISTINCT {select_query} FROM {from_query}'
-        if self.formula.to_sql() != '':
-            query += f' WHERE {self.formula.to_sql()}'
+        if where_query is not None and where_query != '':
+            query += f' WHERE {where_query}'
         return query
 
 
@@ -81,7 +94,8 @@ class Result:
                 for attribute in attributes:
                     output += f'\\text{{{variable.name}.{attribute}}}, '
             else:
-                raise Exception('The selected attributes must consist of a variable and either a attribute as a string or a list of attributes each as a string!')
+                raise Exception(
+                    'The selected attributes must consist of a variable and either a attribute as a string or a list of attributes each as a string!')
         output = output[:-len(', ')]
         output += ']'
         return output
@@ -112,7 +126,11 @@ class Variable(Formula):
         return f'\\text{{{self.name}}} \\in \\text{{{self.type}}}'
 
     def to_sql(self) -> str:
-        pass
+        if self in declared_variables:
+            raise Exception(f'Variable ({self}) was already declared.')
+
+        declared_variables.append(self)
+        return None
 
 
 class Equals(Formula):
@@ -310,13 +328,15 @@ class And(Formula):
         return f'({self.children[0]}\\land {self.children[1]})'
 
     def to_sql(self) -> str:
-        if isinstance(self.children[0], Variable) and isinstance(self.children[1], Variable):
-            return f''
-        if not isinstance(self.children[0], Variable) and isinstance(self.children[1], Variable):
-            return f'{self.children[0].to_sql()}'
-        if isinstance(self.children[0], Variable) and not isinstance(self.children[1], Variable):
-            return f'{self.children[1].to_sql()}'
-        return f'({self.children[0].to_sql()} AND {self.children[1].to_sql()})'
+        left = self.children[0].to_sql()
+        right = self.children[1].to_sql()
+        if left is None and right is None:
+            return None
+        if left is None and right is not None:
+            return right
+        if left is not None and right is None:
+            return left
+        return f'({left} AND {right})'
 
 
 class Or(Formula):
@@ -327,13 +347,15 @@ class Or(Formula):
         return f'({self.children[0]} \\lor {self.children[1]})'
 
     def to_sql(self) -> str:
-        if isinstance(self.children[0], Variable) and isinstance(self.children[1], Variable):
-            return f''
-        if not isinstance(self.children[0], Variable) and isinstance(self.children[1], Variable):
-            return f'{self.children[0].to_sql()}'
-        if isinstance(self.children[0], Variable) and not isinstance(self.children[1], Variable):
-            return f'{self.children[1].to_sql()}'
-        return f'({self.children[0].to_sql()} OR {self.children[1].to_sql()})'
+        left = self.children[0].to_sql()
+        right = self.children[1].to_sql()
+        if left is None and right is None:
+            return None
+        if left is None and right is not None:
+            return right
+        if left is not None and right is None:
+            return left
+        return f'({left} OR {right})'
 
 
 class Not(Formula):
@@ -349,6 +371,7 @@ class Not(Formula):
 
 # temporary test to check the latex output
 if __name__ == '__main__':
+    '''
     t = Variable(name='t', type='KUNDE')
     a = Variable(name='a', type='KUNDE')
     p = Variable(name='p', type='PET')
@@ -368,3 +391,49 @@ if __name__ == '__main__':
     print(tuple_calculus_2)
     print('')
     print(tuple_calculus_2.to_sql())
+    '''
+
+    bi = Variable('bi', 'Biografie')
+    b = Variable('b', 'Biografie')
+    a = Variable('a', 'Autor')
+    g = Variable('g', 'geschriebenVon')
+
+    expr = And(
+        And(
+            And(
+                And(bi, a),
+                And(
+                    Equals((bi, 'BVorname'), 'Helmut'),
+                    Equals((bi, 'BNachname'), 'Schmidt')
+                )
+            ),
+            Forall(
+                b,
+                Or(
+                    Not(
+                        And(
+                            Equals((b, 'BVorname'), 'Helmut'),
+                            Equals((b, 'BNachname'), 'Schmidt')
+                        )
+                    ),
+                    LessEquals((b, 'Bewertung'),
+                               (bi, 'Bewertung'))
+                )
+            )
+        ),
+        Exists(
+            g,
+            And(
+                Equals((g, 'ISBN'), (bi, 'ISBN')),
+                And(
+                    Equals((g, 'AVorname'), (a, 'Vorname')),
+                    Equals((g, 'ANachname'), (a, 'Nachname'))
+                )
+            )
+        )
+    )
+
+    tupelc = TupleCalculus(
+        Result([], {a: ['Nachname', 'Nationalitaet'], bi: 'Titel'}), expr)
+
+    print(tupelc.to_sql())
